import logging
import os
from datetime import datetime
from glob import glob
from os.path import basename, join
from tempfile import TemporaryDirectory

from ..jpssrdr import atms_sci_to_l0, cris_sci_to_l0, spacecraft_to_l0, viirs_sci_to_l0
from ..rdrgen import packets_to_rdrs

LOG = logging.getLogger(__name__)


def merge_rdrs(rdrs):
    # filter and validiate rdrs
    to_process = {
        (s, p): []
        for s in ("snpp", "j01")
        for p in ("RNSCA-RVIRS", "RCRIS-RNSCA", "RATMS-RNSCA")
    }
    for rdr in rdrs:
        parts = basename(rdr).split("_")
        if len(parts) < 2:
            continue
        product = parts[0]
        # validate product
        if product not in ("RNSCA-RVIRS", "RCRIS-RNSCA", "RATMS-RNSCA"):
            LOG.debug("skipping unsupported product %s", rdr)
            continue
        # validate satellite
        sat = parts[1]
        sat = {"jpss1": "j01", "noaa20": "j01", "npp": "snpp"}.get(sat, sat)
        sat = "j01" if sat in ("jpss1", "noaa20") else sat
        if sat not in ("snpp", "j01"):
            LOG.debug("skipping unsupported satellite %s", sat)
            continue

        to_process[sat, product].append(rdr)

    # use same start/end for all extracting since we don't care about file
    # timestamps
    start = datetime(1970, 1, 1)
    end = datetime.utcnow()

    # do each set of files separately to handle case where different products
    # are provided
    outputs = []
    for sat, product in to_process.keys():
        inputs = to_process[sat, product]
        if not inputs:
            continue

        sci_to_l0 = {
            "RNSCA-RVIRS": viirs_sci_to_l0,
            "RCRIS-RNSCA": cris_sci_to_l0,
            "RATMS-RNSCA": atms_sci_to_l0,
        }[product]

        with TemporaryDirectory(dir=os.getcwd()) as tmpdir:
            # extract RDRs separately, name them after RDR
            for fpath in inputs:
                # extract the SCI data
                fname = basename(fpath)
                pds = sci_to_l0(sat, [fpath], start, end, tmpdir)
                os.rename(pds, join(tmpdir, fname + ".sci.pds"))
                # extract the spacecraft data
                scs = spacecraft_to_l0(sat, [fpath], start, end, tmpdir)
                for idx, pds in enumerate(scs):
                    os.rename(pds, join(tmpdir, fname + ".sc%d.pds" % idx))

            # rdrs contains only file names
            rdrs = packets_to_rdrs(
                sat, glob(join(tmpdir, "*.pds")), aggr_type="full", output_dir=tmpdir
            )
            assert len(rdrs) == 1, "Should have gotten a single RDR"

            # copy back to CWD
            os.rename(join(tmpdir, rdrs[0]), rdrs[0])
            outputs.append(rdrs[0])

    return outputs


def main():
    import argparse

    parser = argparse.ArgumentParser()
    parser.add_argument("rdr", nargs="+", help="Any science rdr (RVIRS, RCRIS, RATMS)")
    args = parser.parse_args()

    if not args.rdrs:
        parser.exit(1, "no RDR's provided")

    for o in merge_rdrs(args.rdr):
        LOG.info("created %s", o)


if __name__ == "__main__":
    main()
